@@ -37,6 +37,10 @@
 #include "x86-linux-setup.h"
 #include "../../kexec/kexec-syscall.h"
 
+#ifndef VIDEO_CAPABILITY_64BIT_BASE
+#define VIDEO_CAPABILITY_64BIT_BASE (1 << 1)	/* Frame buffer base is 64-bit */
+#endif
+
 void init_linux_parameters(struct x86_linux_param_header *real_mode)
 {
 	/* Fill in the values that are usually provided by the kernel. */
@@ -232,8 +236,14 @@ static int parse_edd_extensions(const char *dir, struct edd_info *edd_info)
 	char line[1024];
 	uint16_t flags = 0;
 	FILE *fp;
+	int ret;
+
+	ret = snprintf(filename, PATH_MAX, "%s/%s", dir, "extensions");
+	if (ret < 0 || ret >= PATH_MAX) {
+		fprintf(stderr, "snprintf failed: %s\n", strerror(errno));
+		return -1;
+	}
 
-	snprintf(filename, PATH_MAX, "%s/%s", dir, "extensions");
 	filename[PATH_MAX-1] = 0;
 
 	fp = fopen(filename, "r");
@@ -269,8 +279,14 @@ static int read_edd_raw_data(const char *dir, struct edd_info *edd_info)
 	FILE *fp;
 	size_t read_chars;
 	uint16_t len;
+	int ret;
+
+	ret = snprintf(filename, PATH_MAX, "%s/%s", dir, "raw_data");
+	if (ret < 0 || ret >= PATH_MAX) {
+		fprintf(stderr, "snprintf failed: %s\n", strerror(errno));
+		return -1;
+	}
 
-	snprintf(filename, PATH_MAX, "%s/%s", dir, "raw_data");
 	filename[PATH_MAX-1] = 0;
 
 	fp = fopen(filename, "r");
